
/*
 *                                           
 * this function sets the ALCD of the phy      
 *                                             
 * Active link cable diagnostic: 
 * offers a passive method to estimate the cable length present during an active link. 
 * It uses passive digital signal processing based on adapted data,
 * thus enabling the measurement of cable length with an active link partner.
 * The ALCD cable length measurement accuracy is 5 m
 * for the pair used in the Rx path (as a result of the passive nature of the test, only the pair on the Rx path is measured).
 * To perform an ALCD test, do the following:
 * 1. Ensure the device is currently under a 100BT link. 
 * 2. If the user has performed TDR before performing an ALCD test, the user is advised to reset the device. 
 *
 */
 

#include < stdio.h >
#include <math.h> 
#include "registers.h"
#include "loopback.h"
#include "read_write.h"
#include "Resource files/et1000.h"



int ALCD(int phy_id, int *result_alcd)
{

    int  error=0;
	int pair_swap;
    int physr_data=0;
	int cdcr_data=0;
	int cdsr_data=0;
	int dsacr2_data = 0;
	int cdptc1r_data = 0;
	int cdptc2r_data = 0;
	int cdrr_data = 0;
	long timenow = 0;
	int poll;
	int result = 0;
    double result_alcd_double = 0;
	const double Aval = 0.795;    //coeff mode 
    const double Bval = -22.7;    //coeff mode


	// read register PHYSR mdi or mdix
	physr_data = phy_read(phy_id, PHYSR); 
	if (physr_data < 0)// error
	{
		return error = physr_data;
	}

    pair_swap = physr_data & 0x0100;    // only bit 8
    pair_swap = pair_swap >> 8; //


	//Selects channel for Cable Diagnostics Test
	cdcr_data = phy_read(phy_id, CDCR); 
	if (cdcr_data < 0)// error
	{
		return error = cdcr_data;
	}
    cdcr_data &= 0xFFFE ;   // clear bit 0
	cdcr_data |=  !pair_swap; 
    error = phy_write(phy_id, CDCR, cdcr_data); //write
    if (error < 0)
	{
	     return error;
	}

	//cable diagnostic result select == ALCD
	cdcr_data = phy_read(phy_id, CDCR); 
	if (cdcr_data < 0)// error
	{
		return error = cdcr_data;
	}
    cdcr_data &= 0xF0FF;   // clear bits 11-8
	cdcr_data |= 0x0600; 
    error = phy_write(phy_id, CDCR, cdcr_data); //write
    if (error < 0)
	{
	     return error;
	}

	//cable diagnostic mode select == ALCD
	cdsr_data = phy_read(phy_id, CDSR); 
	if (cdcr_data < 0)// error
	{
		return error = cdcr_data;
	}
    cdsr_data &= 0xFFCF;   // clear bits 5,4
    error = phy_write(phy_id, CDSR, cdsr_data); //write
    if (error < 0)
	{
	     return error;
	}


	//cancel tap DSACR2
    dsacr2_data = 0x1F01;   // clear bits 5,4
    error = phy_write(phy_id, DSACR2, dsacr2_data); //write DSA Configuration Register 2
    if (error < 0)
	{
	     return error;
	}


    //Enable Cable diagnostic pre test configuration
    cdptc1r_data = 0x0100;   // set bit 8
    error = phy_write(phy_id, CDPTC1R, cdptc1r_data); //write CD Pre Test Configuration Control 1 
    if (error < 0)
	{
	     return error;
	}


    //Cable Diagnostic pre test configuration is latched
    cdptc2r_data = 0x0348;   // 
    error = phy_write(phy_id, CDPTC2R, cdptc2r_data); //write CD Pre Test Configuration Control 2 
    if (error < 0)
	{
	     return error;
	}


     Sleep(500);

	//Start ALCD test. 
	cdcr_data = phy_read(phy_id, CDCR); 
	if (cdcr_data < 0)// error
	{
		return error = cdcr_data;
	}
    cdcr_data |= 0x2000; 	// set bit 13
    error = phy_write(phy_id, CDCR, cdcr_data); //write
    if (error < 0)
	{
	     return error;
	}



    poll = polling(phy_id, CDSR, 15, 1500); // continue even if timeout	

	cdrr_data = phy_read(phy_id, CDRR); 
	if (cdrr_data < 0)// error
	{
		return error = cdrr_data;
	}

	if (cdrr_data == 0)// error
	{
		//printf("no info");
		*result_alcd = -1;
	}
    else // cdrr_data >0
	{
		printf ("cdrr_data is %d \n", cdrr_data); 
		          //  txtCableLen = Aval * ((2 ^ 16) - RegVal) + Bval
          result_alcd_double = Aval *(65536 - cdrr_data) + Bval;
          result = (int) (result_alcd_double + 0.5);
		  if (result >= 0)
		  {
		    *result_alcd = result;
		  }
		  else
		  {
	        *result_alcd = -1;	
		  }
	}
 




		return 0;
}



