
/*
 *
 *  TDR (TDR.c): 
 *  The TLK100 uses time domain reflectometry (TDR) to determine the quality   
 *  Of the cables, connectors, and terminations in addition to estimating the 
 *  Cable Length.
 *  Some of the possible problems that can be diagnosed include opens, shorts,   
 *  Cable impedance mismatches, bad connectors, termination mismatches, and   
 *  Any discontinuities on the cable.
 *  The TLK100 device transmits a test pulse of known amplitude (1 V) down  
 *  Each of the two pairs of an attached cable. The transmitted signal continues   
 *  Down the cable and reflects from each cable imperfection, fault, bad   
 *  Connector and the end of the cable itself. After the pulse transmission is 
 *  Completed, the TLK100 measures the return time and amplitude of all these    
 *  Reflected pulses. This Technique enables measuring the distance and     
 *  Magnitude (impedance) of non-terminated cables (open or short),
 *  Discontinuities (bad connectors), and improperly-terminated cables.
 *
 *  Run a TDR test while there is no link and when you suspect a cable fault   
 *  Exists. The TDR measurement accuracy is 1 m when using averaging 
 *  (Note that using the averaging process takes up to 30 seconds) or
 *   2 m without averaging (requires up to 10 seconds).
 *
 */

#include < stdio.h >
#include <math.h> 
#include "registers.h"
#include "TDR.h"
#include "read_write.h"
#include "Resource files/et1000.h" 



int TDR(int phy_id, int *reflection, int *impedance,int averaging)
{

	int error, i,j;
    int reflection_average[10];


	switch(averaging)
	{
    case DISABLE_AVG:
	     error = TDR_run_one_test(phy_id, reflection, impedance);
    break;
    
	case ENABLE_AVG:
	     error = TDR_run_one_test(phy_id, reflection_average, impedance);
		 for (i = 0; i < 10 ; i++)
		 {
           reflection[i] = reflection_average[i];
		 }
		 for (i = 0; i < 5 ; i++)
		 {
	          error = TDR_run_one_test(phy_id, reflection_average, impedance);
		      for (j = 0; j < 10 ; j++)
			  {
				  if (reflection[j] >= 0) //check if the cell holds a value 
				  {
                     if (reflection_average[j] >= 0)
					 {
                          reflection[j] += reflection_average[j];
					 }
				  }
			  }
		 }
		 //divide the cells with numerical > 0 value by 6
		 for (i = 0; i < 10 ; i++)
		 {
			 if (reflection[i] >= 0)
			 {
                 reflection[i] = reflection[i]/6;
			 }
		 }

    break;

	default:
	     error = TDR_run_one_test(phy_id, reflection, impedance);
    break;

	}
 	return error;

}

int TDR_run_one_test(int phy_id, int *reflection, int *impedance)
{

    int  error=0;
	int  i,j,k,poll;
	int  chanNum = 2;
    int  count = 9;
	int  tdrcr_data=0;
	int  tdrlt3_data=0;
	int  tdrlt4_data=0;
	int  tdrht1_data=0;
	int  tdrht2_data=0;
	int  tdrht3_data=0;
	int  tdrht4_data=0;
	int  tdrlcr1_data=0;
	int  tdrspp_data=0; 
    int  cdcr_data=0;
    int  cdsr_data=0;
	int  cdrr_data=0;
	int  tdrsmr_data=0;
	int  tdrpar_data=0;
	int  flag_short_cable;
    int  flag_regval_0;
	const double propagation_delay = 4.64;    //
    const double propagation_offset = 20.35;  //
	const double propagation_offset_short_cables = 22.62; //
	int power_int;
	double power;
    long Locat;
    double Amp;
	//long Amp;
	long RegVal;
	double G;
	double Refl;
    //17_10_10
	int reflection_temp;
	int impedance_temp;


	//set TDR defaults
    tdrcr_data = 0x11ec; //Number of averaging cycles:64 //Number of forward samples for peak detection:5 // comparison.Number of backward samples for peak detection comparison: 4//
	error = phy_write(phy_id, TDRCR, tdrcr_data); //write TDR Control Register
	    if (error < 0)
		{			   
	     return error;
		}
    tdrlt3_data = 0x0506; //Peak (absolute) low threshold value 5, for TX pattern = 5h. //Peak (absolute) low threshold value 4, for TX pattern = 6h//
	error = phy_write(phy_id, TDRLT3, tdrlt3_data); //write TDR Low Threshold Register
	    if (error < 0)
		{			   
	     return error;
		}
    tdrlt4_data = 0x0304; //Peak (absolute) low threshold value 7, for TX pattern = 3h. //Peak (absolute) low threshold value 6, for TX pattern = 4h.//
	error = phy_write(phy_id, TDRLT4, tdrlt4_data); //write TDR Low Threshold Register
	    if (error < 0)
		{			   
	     return error;
		}
    tdrht1_data = 0x2830; //Peak (absolute) High threshold value 1, for TX pattern = 28h. //Peak (absolute) High threshold value 0, for TX pattern = 30h. //
	error = phy_write(phy_id, TDRHT1, tdrht1_data); //TDR High Threshold Register
	    if (error < 0)
		{			   
	     return error;
		}
    tdrht2_data = 0x1f26; //Peak (absolute) High threshold value 3, for TX pattern = 1fh. //Peak (absolute) High threshold value 2, for TX pattern = 26h. 
	error = phy_write(phy_id, TDRHT2, tdrht2_data); //TDR High Threshold Register
	    if (error < 0)
		{			   
	     return error;
		}
    tdrht3_data = 0x141e; //Peak (absolute) High threshold value 5, for TX pattern = 14h. //Peak (absolute) High threshold value 4, for TX pattern = 1eh. //
	error = phy_write(phy_id, TDRHT3, tdrht3_data); //TDR High Threshold Register
	    if (error < 0)
		{			   
	     return error;
		}
    tdrht4_data = 0x0c0f; //Peak (absolute) High threshold value 7, for TX pattern = ch. //Peak (absolute) High threshold value 6, for TX pattern = fh. //
	error = phy_write(phy_id, TDRHT4, tdrht4_data); //TDR High Threshold Register
	    if (error < 0)
		{			   
	     return error;
		}
    tdrlcr1_data = 0x035c; //Forward shadow area from peak detection increment factor (X/128) = 1h. //Forward shadow area from peak detection initial samples size = ah. //Initial skip (ignore) samples number from Tx start = 1ch. //
	error = phy_write(phy_id, TDRLCR1, tdrlcr1_data); //TDR Pattern Control Register 1
	    if (error < 0)
		{			   
	     return error;
		}

		
		//


		for(i=chanNum; i>0; i-- ) // do for both channels:pair a and pair b
		{
             flag_short_cable = FALSE;// differentiate between peaks: when first peak <10 m. to following peaks
             flag_regval_0 = FALSE;   //  differentiate between peaks: when first peak = 0. to following peaks


			 	//Selects channel for Cable Diagnostics Test
                  cdcr_data = phy_read(phy_id, CDCR); 
	              if (cdcr_data < 0)// error
				  {
		               return error = cdcr_data;
				  }
                  cdcr_data &= 0xF0FE ;   // clear bit 0 //clear bits (11:8) TDR peak 0 amplitude, TDR peak 0 location} 
	              cdcr_data |=  i-1; //mdi = rx pair (i-1) = 1. //mdi-x = tx pair (i-1) = 0 //
                  error = phy_write(phy_id, CDCR, cdcr_data); //write
                  if (error < 0)
				  {
	                   return error;
				  } 

                 
			  // start test
                 cdcr_data = phy_read(phy_id, CDCR); 
	             if (cdcr_data < 0)// error
				 { 
          		      return error = cdcr_data;
				 } 
                 cdcr_data |= 0x1000; 	// set bit 12
                 error = phy_write(phy_id, CDCR, cdcr_data); //write
                 if (error < 0)
				 {
	                  return error;
				 } 


              // polling
				 poll = polling(phy_id, CDSR, 13, 2000); 
				 if(poll == -1) // timeout
				 //if(polling(phy_id, CDSR, 13, 2000) == -1) // timeout
				 {
					 return -1;
                 }
 
              
              // read test failure
				 cdsr_data = phy_read(phy_id, CDSR); 
	             if (cdsr_data < 0)// error
				 {
		              return error = cdsr_data;
				 }
                 power = pow(2,14);
                 power_int = (int) (power);
	             cdsr_data = (cdsr_data / power_int) & 0x1;
				 if (cdsr_data ==1)
                 {
					 //17_10_10
					 reflection[count] = -7; // tdr fail
					 impedance[count] = 6; // null
                     if ( i == 2) //ch_b
					 {
                        count = 4;
					 }

					 goto ErrHandler; //   Fail
                 }
                 

               //read and calculate data
				 for(j=4; j>=0; j-- ) // read 5 peaks
                 {

			         //Selects channel for Cable Diagnostics Test
                       cdcr_data = phy_read(phy_id, CDCR); 
	                   if (cdcr_data < 0)// error
					   {
		                    return error = cdcr_data;
					   }
                       cdcr_data &= 0xF0FF ; //clear bits (11:8)  
	                   cdcr_data |= j*0x0100; //TDR peak j amplitude, TDR peak j location  
                       error = phy_write(phy_id, CDCR, cdcr_data); //write
                       if (error < 0)
					   {
	                        return error;
					   } 
                       
  
					   RegVal = phy_read(phy_id, CDRR); 
                       if (RegVal < 0)// error
					   { 
		                    return error = cdrr_data;
					   }

    
                       if (RegVal !=0 && !flag_regval_0)//
					   {
	                        Locat = RegVal & 0xFF ; //location of peak
							power = pow(2,8);        //   
                            power_int = (int) (power);
	                        Amp = (RegVal / power_int) & 0xFF; //impedance of peak
                            
							
						  //do signed
							if (Amp > 127)
							{
							     Amp = Amp - 256;
							}

						  //Calculate Reflection and Impedance
                            if (!flag_short_cable)
							{
								 Refl =  Locat * 8 / propagation_delay /2 - propagation_offset;
							}
                            else
							{
							     Refl = Locat * 8 / propagation_delay /2 - propagation_offset_short_cables;
							}
                            
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
							//short cables < 16 m and regval <>0 1st decision branch
							if (Refl < 16 && RegVal != 0)
							{
								    if (!flag_short_cable) // peak 0 of a short cable
									{
									   //17_10_10 change high thresholds for short cables
                                         tdrht1_data = 0x2424; //Peak (absolute) High threshold value 1, for TX pattern = 28h. //Peak (absolute) High threshold value 0, for TX pattern = 30h. //
	                                     error = phy_write(phy_id, TDRHT1, tdrht1_data); //TDR High Threshold Register
	                                     if (error < 0)
										 {			   
	                                        return error;
										 }
                                         tdrht2_data = 0x1f24; //Peak (absolute) High threshold value 3, for TX pattern = 1fh. //Peak (absolute) High threshold value 2, for TX pattern = 26h. 
	                                     error = phy_write(phy_id, TDRHT2, tdrht2_data); //TDR High Threshold Register
	                                     if (error < 0)
										 {			   
	                                        return error;
										 }										
										
																																															
										tdrsmr_data = 0x8000; //1 = Configure TDR state machine mode. This bit is cleared when TDR is complete
	                                     error = phy_write(phy_id, TDRSMR, tdrsmr_data); //TDR State Machine Enable Register
	                                     if (error < 0)
										 {			    
	                                          return error;
										 }
										 
                                         tdrlcr1_data = 0x035d; //
	                                     error = phy_write(phy_id, TDRLCR1, tdrlcr1_data); //TDR Pattern Control Register 1
	                                     if (error < 0)
										 {			    
	                                          return error;
										 }
                                         
										 Sleep(200);
 
										 for(k=1; k<16; k++)
										 {

                                              tdrpar_data = 0x0; // transmit 0
	                                          error = phy_write(phy_id, TDRPAR, tdrpar_data); //TDR Pattern Amplitude Register
	                                          if (error < 0)
											  {			    
	                                          return error;
											  }
										 }
										 
                                         tdrpar_data = 0x8; // transmit 8
	                                     error = phy_write(phy_id, TDRPAR, tdrpar_data); //TDR Pattern Amplitude Register
	                                     if (error < 0)
										 {			    
	                                          return error;
										 }
											 
										 Sleep(1000);

									  // start test
                                         cdcr_data = phy_read(phy_id, CDCR); 
	                                     if (cdcr_data < 0)// error
										 { 
          		                              return error = cdcr_data;
										 }  
                                         cdcr_data |= 0x1000; 	// set bit 12
                                         error = phy_write(phy_id, CDCR, cdcr_data); //write
                                         if (error < 0)
										 { 
	                                          return error;
										 }  

										               // polling
			                        	 poll = polling(phy_id, CDSR, 13, 10000); 
			                         	 if(poll == -1) // timeout
			                         	 //if(polling(phy_id, CDSR, 13, 2000) == -1) // timeout
										 {
				                         	 return -1;
										 }

                      
                                      // read test failure
				                         cdsr_data = phy_read(phy_id, CDSR); 
	                                     if (cdsr_data < 0)// error
										 { 
		                                      return error = cdsr_data;
										 } 
                                         power =	pow(2,14);
                                         power_int = (int) (power);
	                                     cdsr_data = (cdsr_data / power_int) & 0x1;
				                         if (cdsr_data ==1)
										 { 
                                           //17_10_10
					                       reflection[count] = -7; // tdr fail
					                       impedance[count] = 6; // null
					                       goto ErrHandler; //   Fail

										 } 

                                         RegVal = phy_read(phy_id, CDRR); 
                                         if (RegVal < 0)// error
										 {  
		                                      return error = cdrr_data;
										 } 
						                 
										 flag_short_cable = TRUE;
                                    }
								    else // flag_short_cable = true --> not the first peak (peak 0) 
									{
                                         RegVal = phy_read(phy_id, CDRR); 
                                         if (RegVal < 0)// error
										 {  
		                                      return error = cdrr_data;
										 } 

									}


                                   if (RegVal !=0 )//
									{
	                                     Locat = RegVal & 0xFF ; //location of peak
							             power = pow(2,8);        //   
                                         power_int = (int) (power);
	                                     Amp = (RegVal / power_int) & 0xFF; //impedance of peak
							
               						  //do signed
			             				if (Amp > 127)
										{
							                 Amp = Amp - 256;
										}

						             //Calculate Reflection
						        	   Refl = Locat * 8 / propagation_delay /2 - propagation_offset_short_cables;
								       reflection[count] = (int)(Refl + 0.5);
									} 
					                else // RegVal ==0 --> no info
									{ 
                                         reflection[count] = -2; // <3m 
 
									} 


									   //17_10_10 set back high thresholds for previous value
                                         tdrht1_data = 0x2830; //Peak (absolute) High threshold value 1, for TX pattern = 28h. //Peak (absolute) High threshold value 0, for TX pattern = 30h. //
	                                     error = phy_write(phy_id, TDRHT1, tdrht1_data); //TDR High Threshold Register
	                                     if (error < 0)
										 {			   
	                                        return error;
										 }
                                         tdrht2_data = 0x1f26; //Peak (absolute) High threshold value 3, for TX pattern = 1fh. //Peak (absolute) High threshold value 2, for TX pattern = 26h. 
	                                     error = phy_write(phy_id, TDRHT2, tdrht2_data); //TDR High Threshold Register
	                                     if (error < 0)
										 {			   
	                                        return error;
										 }										
					

                                   } //(Refl < 16 && regval != 0) 
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////                       

                          //fix the smearing
						    if ( 10 <= Refl && Refl < 20  )
							{
								Refl = Refl + 1.3;
							}
						    if ( 20 <= Refl && Refl < 40  )
							{
								Refl = Refl + 1.2;
							}
						    if ( 40 <= Refl && Refl < 60  )
							{
								Refl = Refl + 1;
							}
						    if ( 60 <= Refl && Refl < 80  )
							{
								Refl = Refl - 0.3;
							}
						    if ( 80 <= Refl && Refl < 100  )
							{
								Refl = Refl - 0.3;
							}
						    if ( 100 <= Refl && Refl < 120  )
							{
								Refl = Refl - 0.3;
							}
						    if ( 120 <= Refl && Refl < 150  )
							{
								Refl = Refl - 1;
							}

							if ( 10 <= Refl)
							{
                                reflection[count] = (int) (Refl +0.5);
							}
                          //end of smearing fix
                          
                         // impedance
                            G = Amp / GetPeakAmpRef(Refl);
                
                            if (RegVal != 0)
							{
                                 if (1.3 <= G || G <= 0.9) 
								 {
                                      impedance[count] = (int) (100 * (1 + G) / (1 - G) + 0.5);
								 }
                                 else
								 {
                                      G = 1;
								 }
							

                                 if (impedance[count] > 200 || G == 1) // Open
								 {
                                      impedance[count] = 1;  //open
								 }
								 //17_10_10
                                 else if (impedance[count] < 40) // Short or low
								 {
                                      impedance[count] = 2; // Short or low
								 }								 
                                 else if (impedance[count] >= 40 && impedance[count] < 75)
								 { 
                                      impedance[count] = 3; //bad
								 } 
                                 else
								 { 
                                      impedance[count] = 4; //terminated
								 } 
                            }                                                                          
                            else // regval = 0   for case where the program pass through the short pulse and then regval = 0
							{
                                 impedance[count] = 5; //"No Info"
							}

                       }
                       else if ( j == 4) //   if !(RegVal !=0 && !flag_regval_0)						   
					   {
					    //  regval = 0, only for last peak  read only peak 0 and decide                      
                         // prepare special pulse. c0b = 350h and width 0x02 * 5

                            tdrsmr_data = 0x8000; //1 = Configure TDR state machine mode. This bit is cleared when TDR is complete
	                        error = phy_write(phy_id, TDRSMR, tdrsmr_data); //TDR State Machine Enable Register
                            if (error < 0)
							{			    
	                             return error;
							}

                            tdrlcr1_data = 0x0350; //
	                        error = phy_write(phy_id, TDRLCR1, tdrlcr1_data); //TDR Pattern Control Register 1
	                        if (error < 0)
							{			    
	                             return error;
							}
                                         
						    Sleep(200);

							for(k=1; k<12; k++)
							{

                                 tdrpar_data = 0x0; // transmit amplitude 0
                                 error = phy_write(phy_id, TDRPAR, tdrpar_data); //TDR Pattern Amplitude Register
	                             if (error < 0)
								 {			    
	                                  return error;
								 }
							}

							for(k=1; k<6; k++)			 
                            {
                                 tdrpar_data = 0x1; // transmit amplitude 1
	                             error = phy_write(phy_id, TDRPAR, tdrpar_data); //TDR Pattern Amplitude Register
	                             if (error < 0)
								 {			    
	                                  return error;
								 }
							}
							
							Sleep(1000);

						 // start test
                            cdcr_data = phy_read(phy_id, CDCR); 
	                        if (cdcr_data < 0)// error
							{ 
          		                 return error = cdcr_data;
							}
							
                            cdcr_data |= 0x1000; 	// set bit 12
                            error = phy_write(phy_id, CDCR, cdcr_data); //write
                            if (error < 0)
							{ 
	                             return error;
							}  

                         // polling
				            if(polling(phy_id, CDSR, 13, 2000) == -1) // timeout
							{ 
				                 return -1;
							} 
                      
                        //  read test failure
				            cdsr_data = phy_read(phy_id, CDSR); 
	                        if (cdsr_data < 0)// error
							{ 
		                        return error = cdsr_data;
							} 
                            power = pow(2,14);
                            power_int = (int) (power);
	                        cdsr_data = (cdsr_data / power_int) & 0x1;
				            if (cdsr_data ==1)
							{
							 //17_10_10	
					         reflection[count] = -7; // tdr fail
					         impedance[count] = 6; // null
					         goto ErrHandler; //   Fail

							} 

                            RegVal = phy_read(phy_id, CDRR); 
                            if (RegVal < 0)// error
							{  
		                         return error = cdrr_data;
							} 

						    power = pow(2,8);        //   
                            power_int = (int) (power);
	                        Amp = (RegVal / power_int) & 0xFF; //impedance of peak

                            if (Amp >= 25 && Amp < 60)
							{
                                 reflection[count] = -2; // <3m						   
                                 impedance[count] = 1;  //open
							}
                            else if (Amp < 25 && Amp > 18)
							{
                                 reflection[count] = -3; //"Terminated"
                                 impedance[count] = 6; // null
							}
                            else //short
							{
                                 reflection[count] = -2; // <3m
                                 impedance[count] = 2; //"Short or low"
							}
                           
                            tdrlcr1_data = 0x035c; //
	                        error = phy_write(phy_id, TDRLCR1, tdrlcr1_data); //TDR Pattern Control Register 1
	                        if (error < 0)
							{			    
	                             return error;
							}
     
							flag_regval_0 = TRUE;

					   } //else if ( j == 4) // -->  if (RegVal !=0 && !flag_regval_0)// 

                       else // peak 3,2,1,0
					   {
                            reflection[count] = -6; // "No Info"
                            impedance[count] = 5; //"No Info"
					   }

                   count = count -1;
         
				} //for(j=4; j>=0; j-- ) // read 5 peaks	   

                ErrHandler:;
		} //for(i=chanNum; i>0; i-- ) // do for both channels:pair a and pair b


    //for case when peak 0 and peak 4 are identical
      if (reflection[9] == reflection[5])
	  {
            reflection[5] = -6;//"No Info"
            impedance[5] =  5;//"No Info"
	  }
      if (reflection[4] == reflection[0])
	  {
            reflection[0] = -6;//"No Info"
            impedance[0] =  5;//"No Info"
	  }
         
//17_10_10 new peak sorting, for cases when the last peak reflection is shorter than 1 of the previous peaks     
	  //ch_b
   if (reflection[9] > 0) // do sorting only for real numbers
   {
   for(count = 8; count >= 5; count-- ) // cycle through all peaks of channel b  
   {
      if (reflection[count] > reflection[9])
	  {
          reflection_temp = reflection[9];
          impedance_temp = impedance[9]; 
          reflection[9] = reflection[count];
          impedance[9] = impedance[count];
          reflection[count] = reflection_temp;
          impedance[count] = impedance_temp;
	  }
   }
   }

	  //ch_a
   if (reflection[4] > 0) // do sorting only for real numbers
   {
   for(count = 3; count >= 0; count-- ) // cycle through all peaks of channel a  
   {
      if (reflection[count] > reflection[4])
	  {
          reflection_temp = reflection[4];
          impedance_temp = impedance[4]; 
          reflection[4] = reflection[count];
          impedance[4] = impedance[count];
          reflection[count] = reflection_temp;
          impedance[count] = impedance_temp;

	  }
   }
   }
	  
	  
	  
	  
	  
	  
	  
	  
	  
	  
return 1;  // Passed

}





int GetPeakAmpRef(double CabLength)
{

	double MinLength, MaxLength;
    int ind;
    int output;

    MinLength = 1;
    MaxLength = 150;
    
    if (CabLength < MinLength)
	{
	 CabLength = MinLength;
    }
    if (CabLength > MaxLength)
	{
	 CabLength = MaxLength;
    }
       
    ind = (int) (CabLength +0.5);


	  if (ind <= 4)
	  {
		  output =  141;
	  }
	  //17_10_10
      else if (5<= ind && ind <=10)
	  {
		  output =  140;
	  }
      else if (11 <= ind && ind <=19)
	  {
		  output =  120;
	  }
 
      else if (20<= ind && ind  <=39)
	  {
		  output =  111;
	  }
      else if (40<= ind && ind <=59)
	  {
		  output =  70;
	  }
      else if (60<= ind && ind <=79)
	  {
		  output =  53;
	  }
     else if (80<= ind && ind <=99)
	  {
		  output =  44;
	  }
     else if (100<= ind && ind <=129)
	  {
		  output =  33;
	  }
     else if (130<= ind && ind <=149)
	  {
		  output =  74;
	  }

     else //150 <= ind Then
	 {
		 output =  22;
	 }
        
    
    // exception for cables that refl = 4.1
       if (4.05<= CabLength && CabLength <= 4.15)
	   {
		  output =  -60;
	   }
	   

	   return output;
}

